#include <stdlib.h>
#include "uart.h"
#include "gba.h"
#include "screenmode.h"
#include "font.h"

void scrWaitVbl();
void scrWaitVSync();
u16 scrIsVbl();

  int uppercase;
  int charpos;
  char *tables[2][11] = {{
//lowercase
	"+&@/4%$89  0    ",
	" -?!,.:7\"'012()1",
	"abc2",
	"def3",
	"ghi4",
	"jkl5",
	"mno6",
	"pqrs7  ",
	"tuv8",
	"wxyz9",
	"#*" },
//uppercase
	{"+&@/4%$89  0    ",
	" -?!,.:7\"'012()1",
	"ABC2",
	"DEF3",
	"GHI4",
	"JKL5",
	"MNO6",
	"PQRS7  ",
	"TUV8",
	"WXYZ9",
	"#*" }};
    
char isKeyStroke(char* data){
	return !memcmp(data,"AT+CKPD",7);
}

char translate(char* data){
	int key;
	int strokes=0;
	int ptr=9;
	do{
		if(strokes==0)
			key=data[ptr];
		if(data[ptr]==key)
			strokes++;
		else
			strokes=0;
		ptr++;
	}while(data[ptr]!='"' && data[ptr]!=13);

    if(strokes>0){
    	if(!memcmp(&data[ptr+1],",20",3)){
			return key; //numerical
		}else if(key >= 0x30 && key <= 0x39){
            return tables[uppercase][key-0x30][strokes-1];
          }else if(key == '#'){
            return tables[uppercase][10][strokes-1];
          }else if(key == '>' || key == '<'){
            return key;
          }else if(key == 'c'){
            return 8;  //backspace
		}else if(key == '*'){
			//Switch lower/upper case
			uppercase=1-uppercase;
			return 0; //not an error condition, but no char to return.
        }else{
			return 0; //error condition, should not occur.
        }
	}else{
		return 0; //error condition, should not occur.
    }
}

void send_byte(char c) {
    while(REG_SIOCNT & SIO_SEND_DATA);
    REG_SIODATA8 = c;
    REG_SIOCNT |= SIO_RECV_DATA | SIO_SEND_DATA;
}

void send_string(const char* str) {
    while(*str) {
        send_byte(*str);
        str++;
    }
}

//writes one character to screen
void typechar(char c){
    if(c){
        if(c==8){ //backspace
            VideoBuffer[0x4000+charpos]=' ';
            charpos--;
            VideoBuffer[0x4000+charpos]='_';
        }else{
            VideoBuffer[0x4000+charpos]=c;
            charpos++;
            VideoBuffer[0x4000+charpos]='_';
        }
    }
}

//writes one string to screen
void typestr(char* data){
int ptr=0;
    while(data[ptr]!=0){
        typechar(data[ptr]);
        ptr++;
    }
}

char getchar(void){
    char uartbyte;
    while(REG_SIOCNT & SIO_RECV_DATA);
    uartbyte = REG_SIODATA8;
    REG_SIOCNT |= SIO_RECV_DATA | SIO_SEND_DATA;
    return uartbyte;
}

void AgbMain(void){
    int loop, x_loop, y_loop;
	char text[] = {'O', 'K', 13, 10, 0};
	int y;
	char h[100];
    uppercase=0;

    //Init UART                     
    REG_RCNT = 0;
    REG_SIOCNT = 0;
    REG_SIOCNT = SIO_BAUD_9600 | SIO_LENGTH_8 | SIO_SEND_ENABLE | SIO_RECV_ENABLE | SIO_USE_UART;
    REG_SIOCNT |= SIO_RECV_DATA | SIO_SEND_DATA;
    //UART initialized

    //font palette
    for(loop = 0; loop < fontPalLen; loop++)
	    BGPaletteMem[loop] = fontPal[loop];
    //font characters
    for(loop=0; loop<fontDataLen; loop++)
        VideoBuffer[loop]=fontData[loop];
    //fill entire screen with spaces
    for(y_loop=0; y_loop<20; y_loop++)
        for (x_loop=0; x_loop<30; x_loop++)
            VideoBuffer[0x4000+x_loop+32*y_loop]=' ';
    
    SetMode(MODE_0 | BG0_ENABLE);
    REG_BG0CNT = BG_COLOUR_256x1 | BG_T256x256 |BG_MAPBASE(0x8000) | BG_CHARBASE(0x0000) | BG_PRIORITY(3);

    charpos=64;
    typestr("Keyboard Advance test by AmPz ");
    charpos=128;
    typestr("  No keyboard detected...     ");
    loop=0;
	while(loop!=2){
          y=0;
          //read one line from keyboard
		do{
                h[y++]=getchar();
		}while(h[y-1]!=13);
          //acknowledge
          send_string(text);
          if(loop==0){
              charpos=128;
              typestr("  Keyboard detected!          ");
              charpos=160;
              typestr("  Please press the SMS key... ");
          }
          if(!memcmp(h,"AT*ESVM=",8)){
              loop++;
              charpos=160;
              if(loop==1)
                  typestr("  Please wait a second...     ");
          }
    }
    charpos=128;
    typestr("  Keyboard initialized.        ");
    charpos=160;
    typestr("Use shift+space for caps lock  ");
    charpos=192;

	while(1){
          y=0;
          //read one line from keyboard
		do{
                h[y++]=getchar();
		}while(h[y-1]!=13);
          //acknowledge
          send_string(text);

		if(isKeyStroke(h)){
              typechar(translate(h));
		}
	}
}

// Wait until vertical blank
void scrWaitVbl(){
	while( !scrIsVbl() );
}
// Wait for NEXT vertical blank
void scrWaitVSync(){
	while( scrIsVbl() );
	while( !scrIsVbl() );
}

// Is the screen in vertical blank?
u16 scrIsVbl(){
	return( REG_VCOUNT > 160 );
}

